# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/ids_finder',
                'doc_host': 'https://Beforerr.github.io',
                'git_url': 'https://github.com/Beforerr/ids_finder',
                'lib_path': 'discontinuitypy'},
  'syms': { 'discontinuitypy.core.detection': { 'discontinuitypy.core.detection._compute_indices': ( 'ids_detection.html#_compute_indices',
                                                                                                     'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.add_neighbor_std': ( 'ids_detection.html#add_neighbor_std',
                                                                                                     'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.compute_combinded_std': ( 'ids_detection.html#compute_combinded_std',
                                                                                                          'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.compute_index_diff': ( 'ids_detection.html#compute_index_diff',
                                                                                                       'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.compute_index_fluctuation': ( 'ids_detection.html#compute_index_fluctuation',
                                                                                                              'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.compute_index_std': ( 'ids_detection.html#compute_index_std',
                                                                                                      'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.compute_indices': ( 'ids_detection.html#compute_indices',
                                                                                                    'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.compute_std': ( 'ids_detection.html#compute_std',
                                                                                                'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.detect_events': ( 'ids_detection.html#detect_events',
                                                                                                  'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.filter_indices': ( 'ids_detection.html#filter_indices',
                                                                                                   'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.pl_dvec': ( 'ids_detection.html#pl_dvec',
                                                                                            'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.pl_format_time': ( 'ids_detection.html#pl_format_time',
                                                                                                   'discontinuitypy/core/detection.py')},
            'discontinuitypy.core.event': { 'discontinuitypy.core.event.get_event_data': ( 'event.html#get_event_data',
                                                                                           'discontinuitypy/core/event.py')},
            'discontinuitypy.core.pipeline': { 'discontinuitypy.core.pipeline.compress_data_by_cands': ( 'ids_finder.html#compress_data_by_cands',
                                                                                                         'discontinuitypy/core/pipeline.py'),
                                               'discontinuitypy.core.pipeline.extract_features': ( 'ids_finder.html#extract_features',
                                                                                                   'discontinuitypy/core/pipeline.py'),
                                               'discontinuitypy.core.pipeline.ids_finder': ( 'ids_finder.html#ids_finder',
                                                                                             'discontinuitypy/core/pipeline.py')},
            'discontinuitypy.core.propeties': { 'discontinuitypy.core.propeties.IDsPipeline': ( 'ids_properties.html#idspipeline',
                                                                                                'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPipeline.__init__': ( 'ids_properties.html#idspipeline.__init__',
                                                                                                         'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPipeline.calc_duration': ( 'ids_properties.html#idspipeline.calc_duration',
                                                                                                              'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPipeline.calc_mva_features': ( 'ids_properties.html#idspipeline.calc_mva_features',
                                                                                                                  'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPipeline.calc_normal_direction': ( 'ids_properties.html#idspipeline.calc_normal_direction',
                                                                                                                      'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPipeline.calc_rotation_angle': ( 'ids_properties.html#idspipeline.calc_rotation_angle',
                                                                                                                    'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPipeline.calc_vec_change': ( 'ids_properties.html#idspipeline.calc_vec_change',
                                                                                                                'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_candidate_duration': ( 'ids_properties.html#calc_candidate_duration',
                                                                                                            'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_events_normal_direction': ( 'ids_properties.html#calc_events_normal_direction',
                                                                                                                 'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_events_rotation_angle': ( 'ids_properties.html#calc_events_rotation_angle',
                                                                                                               'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_events_vec_change': ( 'ids_properties.html#calc_events_vec_change',
                                                                                                           'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_normal_direction': ( 'ids_properties.html#calc_normal_direction',
                                                                                                          'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_rotation_angle': ( 'ids_properties.html#calc_rotation_angle',
                                                                                                        'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.get_candidate_data': ( 'ids_properties.html#get_candidate_data',
                                                                                                       'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.get_candidates': ( 'ids_properties.html#get_candidates',
                                                                                                   'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.get_data_at_times': ( 'ids_properties.html#get_data_at_times',
                                                                                                      'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.pdp.ApplyToRows._transform': ( 'ids_properties.html#pdp.applytorows._transform',
                                                                                                               'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.process_events': ( 'ids_properties.html#process_events',
                                                                                                   'discontinuitypy/core/propeties.py')},
            'discontinuitypy.datasets': { 'discontinuitypy.datasets.IDsDataset': ( 'datasets.html#idsdataset',
                                                                                   'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.Config': ( 'datasets.html#idsdataset.config',
                                                                                          'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.get_candidate': ( 'datasets.html#idsdataset.get_candidate',
                                                                                                 'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.get_candidate_data': ( 'datasets.html#idsdataset.get_candidate_data',
                                                                                                      'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.plot_candidate': ( 'datasets.html#idsdataset.plot_candidate',
                                                                                                  'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.plot_candidates': ( 'datasets.html#idsdataset.plot_candidates',
                                                                                                   'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.update_candidates_with_plasma_data': ( 'datasets.html#idsdataset.update_candidates_with_plasma_data',
                                                                                                                      'discontinuitypy/datasets.py')},
            'discontinuitypy.datasets_kedro': { 'discontinuitypy.datasets_kedro.E1IDsDataset': ( 'datasets_kedro.html#e1idsdataset',
                                                                                                 'discontinuitypy/datasets_kedro.py'),
                                                'discontinuitypy.datasets_kedro.cIDsDataset': ( 'datasets_kedro.html#cidsdataset',
                                                                                                'discontinuitypy/datasets_kedro.py'),
                                                'discontinuitypy.datasets_kedro.cIDsDataset.__init__': ( 'datasets_kedro.html#cidsdataset.__init__',
                                                                                                         'discontinuitypy/datasets_kedro.py'),
                                                'discontinuitypy.datasets_kedro.cIDsDataset.load_data': ( 'datasets_kedro.html#cidsdataset.load_data',
                                                                                                          'discontinuitypy/datasets_kedro.py'),
                                                'discontinuitypy.datasets_kedro.cIDsDataset.load_events': ( 'datasets_kedro.html#cidsdataset.load_events',
                                                                                                            'discontinuitypy/datasets_kedro.py')},
            'discontinuitypy.integration': { 'discontinuitypy.integration.calc_combined_features': ( 'mag_plasma.html#calc_combined_features',
                                                                                                     'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.calc_rotation_angle_pl': ( 'mag_plasma.html#calc_rotation_angle_pl',
                                                                                                     'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.combine_features': ( 'mag_plasma.html#combine_features',
                                                                                               'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.compute_Alfven_current': ( 'mag_plasma.html#compute_alfven_current',
                                                                                                     'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.compute_Alfven_speed': ( 'mag_plasma.html#compute_alfven_speed',
                                                                                                   'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.compute_inertial_length': ( 'mag_plasma.html#compute_inertial_length',
                                                                                                      'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.vector_project': ( 'mag_plasma.html#vector_project',
                                                                                             'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.vector_project_pl': ( 'mag_plasma.html#vector_project_pl',
                                                                                                'discontinuitypy/integration.py')},
            'discontinuitypy.propeties.duration': { 'discontinuitypy.propeties.duration.calc_d_duration': ( 'properties/duration.html#calc_d_duration',
                                                                                                            'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.calc_duration': ( 'properties/duration.html#calc_duration',
                                                                                                          'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.find_start_end_times': ( 'properties/duration.html#find_start_end_times',
                                                                                                                 'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.get_time_from_condition': ( 'properties/duration.html#get_time_from_condition',
                                                                                                                    'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.ts_max_derivative': ( 'properties/duration.html#ts_max_derivative',
                                                                                                              'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.ts_max_distance': ( 'properties/duration.html#ts_max_distance',
                                                                                                            'discontinuitypy/propeties/duration.py')},
            'discontinuitypy.propeties.mva': { 'discontinuitypy.propeties.mva.calc_candidate_mva_features': ( 'properties/mva.html#calc_candidate_mva_features',
                                                                                                              'discontinuitypy/propeties/mva.py'),
                                               'discontinuitypy.propeties.mva.calc_mva_features': ( 'properties/mva.html#calc_mva_features',
                                                                                                    'discontinuitypy/propeties/mva.py'),
                                               'discontinuitypy.propeties.mva.fit_maxiumum_variance_direction': ( 'properties/mva.html#fit_maxiumum_variance_direction',
                                                                                                                  'discontinuitypy/propeties/mva.py'),
                                               'discontinuitypy.propeties.mva.minvar': ( 'properties/mva.html#minvar',
                                                                                         'discontinuitypy/propeties/mva.py')},
            'discontinuitypy.utils.analysis': { 'discontinuitypy.utils.analysis.filter_before_jupiter': ( 'utils/analysis_utils.html#filter_before_jupiter',
                                                                                                          'discontinuitypy/utils/analysis.py'),
                                                'discontinuitypy.utils.analysis.filter_tranges_ds': ( 'utils/analysis_utils.html#filter_tranges_ds',
                                                                                                      'discontinuitypy/utils/analysis.py'),
                                                'discontinuitypy.utils.analysis.link_coord2dim': ( 'utils/analysis_utils.html#link_coord2dim',
                                                                                                   'discontinuitypy/utils/analysis.py'),
                                                'discontinuitypy.utils.analysis.n2_normalize': ( 'utils/analysis_utils.html#n2_normalize',
                                                                                                 'discontinuitypy/utils/analysis.py')},
            'discontinuitypy.utils.basic': { 'discontinuitypy.utils.basic.DataConfig': ( 'utils/basic.html#dataconfig',
                                                                                         'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic._expand_selectors': ( 'utils/basic.html#_expand_selectors',
                                                                                                'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.calc_vec_mag': ( 'utils/basic.html#calc_vec_mag',
                                                                                           'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.check_fgm': ( 'utils/basic.html#check_fgm',
                                                                                        'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.concat_df': ( 'utils/basic.html#concat_df',
                                                                                        'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.concat_partitions': ( 'utils/basic.html#concat_partitions',
                                                                                                'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.df2ts': ( 'utils/basic.html#df2ts',
                                                                                    'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.filter_tranges': ( 'utils/basic.html#filter_tranges',
                                                                                             'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.filter_tranges_df': ( 'utils/basic.html#filter_tranges_df',
                                                                                                'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.format_timedelta': ( 'utils/basic.html#format_timedelta',
                                                                                               'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.partition_data_by_time': ( 'utils/basic.html#partition_data_by_time',
                                                                                                     'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.partition_data_by_ts': ( 'utils/basic.html#partition_data_by_ts',
                                                                                                   'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.partition_data_by_year': ( 'utils/basic.html#partition_data_by_year',
                                                                                                     'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.partition_data_by_year_month': ( 'utils/basic.html#partition_data_by_year_month',
                                                                                                           'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.pl.DataFrame.plot': ( 'utils/basic.html#pl.dataframe.plot',
                                                                                                'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.pl_norm': ( 'utils/basic.html#pl_norm',
                                                                                      'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.pmap': ( 'utils/basic.html#pmap',
                                                                                   'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.resample': ( 'utils/basic.html#resample',
                                                                                       'discontinuitypy/utils/basic.py')},
            'discontinuitypy.utils.kedro': { 'discontinuitypy.utils.kedro.load_context': ( 'utils/kedro.html#load_context',
                                                                                           'discontinuitypy/utils/kedro.py')},
            'discontinuitypy.utils.lbl': { 'discontinuitypy.utils.lbl.LblDataset': ( 'utils/lbl.html#lbldataset',
                                                                                     'discontinuitypy/utils/lbl.py'),
                                           'discontinuitypy.utils.lbl.LblDataset.__init__': ( 'utils/lbl.html#lbldataset.__init__',
                                                                                              'discontinuitypy/utils/lbl.py'),
                                           'discontinuitypy.utils.lbl.LblDataset._describe': ( 'utils/lbl.html#lbldataset._describe',
                                                                                               'discontinuitypy/utils/lbl.py'),
                                           'discontinuitypy.utils.lbl.LblDataset._load': ( 'utils/lbl.html#lbldataset._load',
                                                                                           'discontinuitypy/utils/lbl.py'),
                                           'discontinuitypy.utils.lbl.LblDataset._save': ( 'utils/lbl.html#lbldataset._save',
                                                                                           'discontinuitypy/utils/lbl.py'),
                                           'discontinuitypy.utils.lbl.load_lbl': ( 'utils/lbl.html#load_lbl',
                                                                                   'discontinuitypy/utils/lbl.py')},
            'discontinuitypy.utils.plot': { 'discontinuitypy.utils.plot.plot_candidate': ( 'utils/plotting.html#plot_candidate',
                                                                                           'discontinuitypy/utils/plot.py'),
                                            'discontinuitypy.utils.plot.savefig': ( 'utils/plotting.html#savefig',
                                                                                    'discontinuitypy/utils/plot.py'),
                                            'discontinuitypy.utils.plot.setup_mva_plot': ( 'utils/plotting.html#setup_mva_plot',
                                                                                           'discontinuitypy/utils/plot.py'),
                                            'discontinuitypy.utils.plot.time_stamp': ( 'utils/plotting.html#time_stamp',
                                                                                       'discontinuitypy/utils/plot.py')},
            'discontinuitypy.utils.polars': { 'discontinuitypy.utils.polars._expand_selectors': ( 'utils/polars.html#_expand_selectors',
                                                                                                  'discontinuitypy/utils/polars.py'),
                                              'discontinuitypy.utils.polars.convert_to_pd_dataframe': ( 'utils/polars.html#convert_to_pd_dataframe',
                                                                                                        'discontinuitypy/utils/polars.py'),
                                              'discontinuitypy.utils.polars.create_partitions': ( 'utils/polars.html#create_partitions',
                                                                                                  'discontinuitypy/utils/polars.py'),
                                              'discontinuitypy.utils.polars.decompose_vector': ( 'utils/polars.html#decompose_vector',
                                                                                                 'discontinuitypy/utils/polars.py'),
                                              'discontinuitypy.utils.polars.pl_norm': ( 'utils/polars.html#pl_norm',
                                                                                        'discontinuitypy/utils/polars.py'),
                                              'discontinuitypy.utils.polars.sort': ( 'utils/polars.html#sort',
                                                                                     'discontinuitypy/utils/polars.py')}}}
