# AUTOGENERATED! DO NOT EDIT! File to edit: ../notebooks/__init__.ipynb.

# %% auto 0
__all__ = ['ROOT_DIR']

# %% ../notebooks/__init__.ipynb 1
__version__ = "0.0.1"

# %% ../notebooks/__init__.ipynb 2
from pathlib import Path

# %% ../notebooks/__init__.ipynb 3
#| eval: false
ROOT_DIR = Path(__file__).parent.parent

# %% ../notebooks/__init__.ipynb 6
# from discontinuitypy.utils.basic import load_params
# PARAMS = load_params()
